package com.fuzamei.web;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.fuzamei.entity.Role;
import com.fuzamei.entity.User;
import com.fuzamei.entity.User.GroupA;
import com.fuzamei.entity.User.GroupB;
import com.fuzamei.service.UserService;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.PageDTO;

/**
 * 
* <b>Description:用户模块Action层</b><br> 
* @Note
* <b>Author:maamin
* @version 1.0
* @Note
* <b>ProjectName:</b> ccb_fund_trusteeship
* <br><b>PackageName:</b> com.fuzamei.web
* <br><b>ClassName:</b> UserAction
* <br><b>Date:</b> 2017年12月25日 上午10:16:00
 */
@RestController
@RequestMapping(value = "/user")
public class UserAction {
	@Resource(name = "userService")
	private UserService userService;
	int page = 1;
	int rows = 10;
	HttpServletRequest request;
	/**
	 * 
	* <b>Description:用户登录方法</b><br> 
	* @param data
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月25日 上午10:18:07
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/login",method = RequestMethod.POST)
	public Map<String,Object> login(@Validated(GroupA.class) @RequestBody User user,BindingResult result){
		System.out.println("进入用户登录login方法……"+user.getAccount());
		Map<String,Object> resultMap = new LinkedHashMap<String,Object>();	//存放最终返回结果
		try {
			if(result.hasErrors()){
				List<FieldError> errors = result.getFieldErrors();
				resultMap = JSONUtil.getJsonMap(12431, false, errors.get(0).getDefaultMessage(),null);
				return resultMap;
			}
			resultMap = userService.login(user);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			resultMap = JSONUtil.getJsonMap(12431, false, "用户登录失败", null);
		}
		System.out.println("用户登录login方式结束……");
		return resultMap;
	}
	/**
	 * 
	* <b>Description:初始化创建所有用户信息</b><br> 
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月25日 上午10:25:24
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/initUserList",method = RequestMethod.GET)
	public Map<String,Object> initUserList(){
		System.out.println("初始化创建用户列表……");
		return userService.initUserList();
	}
	/**
	 * 
	* <b>Description:添加用户信息</b><br> 
	* @param data
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月25日 下午2:58:44
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/addUser",method = RequestMethod.POST)
	public Map<String,Object> addUser(@Validated(GroupB.class) @RequestBody User user,BindingResult result){
		System.out.println("添加用户……");
		Map<String,Object> resultMap = new HashMap<String,Object>();	//存放最终返回结果
		try {
			if(result.hasErrors()){
				resultMap = JSONUtil.getJsonMap(12431, false, result.getFieldErrors().get(0).getDefaultMessage(), null);
				return resultMap;
			}
			resultMap = userService.addUser(user);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			resultMap = JSONUtil.getJsonMap(12431, false, "添加用户失败", null);
		}
		System.out.println("添加用户结束……");
		return resultMap;
	}
	/**
	 * 
	* <b>Description:编辑用户信息</b><br> 
	* @param user
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月27日 上午11:03:08
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/editUser",method = RequestMethod.POST)
	public Map<String,Object> editUser(@Validated(GroupB.class) @RequestBody User user,BindingResult result){
		Map<String,Object> resultMap = new HashMap<String,Object>();
		if(result.hasErrors()){
			resultMap = JSONUtil.getJsonMap(12431, false,result.getFieldErrors().get(0).getDefaultMessage() , null);
			return resultMap;
		}
		try{
			resultMap = userService.editUser(user);
		}catch(Exception e){
//			System.out.println("错误信息："+e.getMessage());
			resultMap = JSONUtil.getJsonMap(12431, false, "编辑用户信息失败", null);
		}
		return resultMap;
	}
	/**
	 * 
	* <b>Description:查询用户信息</b><br> 
	* @param user
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2018年1月9日 上午11:07:46
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/getUsers",method = RequestMethod.POST)
	public Map<String,Object> getUsers(@RequestBody User user){
		System.out.println("查询用户列表信息……");
		Map<String,Object> resultMap = new HashMap<String,Object>();
		PageDTO pageDTO = new PageDTO();
		try {
			pageDTO = userService.getUsers(user,page,rows);
			resultMap = JSONUtil.getJsonMap(200, true, "查询用户信息成功", pageDTO);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			resultMap = JSONUtil.getJsonMap(12431, false, "查询用户信息报错", null);
		}
		return resultMap;
	}
	/**
	 * 
	* <b>Description:获取所有角色信息</b><br> 
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2018年1月10日 下午5:36:10
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/getAllRoles",method = RequestMethod.GET)
	public Map<String,Object> getAllRoles(){
		Map<String,Object> resultMap = new LinkedHashMap<String,Object>();
		List<Role> roles = new ArrayList<Role>();
		try {
			roles = userService.getAllRoles();
			resultMap = JSONUtil.getJsonMap(200, true, "获取角色信息成功", roles);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			resultMap = JSONUtil.getJsonMap(12431, false, "获取角色信息失败", roles);
		}
		return resultMap;
	}
	/**
	 * 
	* <b>Description:通过id查询用户信息</b><br> 
	* @param userId
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2018年1月10日 下午5:36:21
	* <br><b>Version:</b> 1.0
	 */
	@RequestMapping(value = "/findUserById",method = RequestMethod.GET)
	public Map<String,Object> findUserById(@RequestParam(value = "userId") String userId){
		Map<String,Object> resultMap = new LinkedHashMap<String,Object>();
		Map<String,Object> result = new LinkedHashMap<String,Object>();
		try {
			result = userService.findUserById(userId);
			resultMap = JSONUtil.getJsonMap(200, true, "查询用户信息成功", result);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			resultMap = JSONUtil.getJsonMap(12431, false, "查询用户信息失败", result);
		}
		return resultMap;
	}
}
